-- DorHUD by DorentuZ` -- http://steamcommunity.com/id/dorentuz/
--   Hitmarkers module by BangL
--
-- This Source Code Form is subject to the terms of the Mozilla Public
-- License, v. 2.0. If a copy of the MPL was not distributed with this
-- file, You can obtain one at http://mozilla.org/MPL/2.0/.

local module = D:module('hitmarkers')

HUDManager.__name = "HUDManager"

module:hook(HUDManager, "init", function(self)
	module:call_orig(HUDManager, "init", self)
	self:setup_dorhud_hitmarkers_workspace()
end)

module:hook(HUDManager, "setup_dorhud_hitmarkers_workspace", function(self)
	if self._dorhud_hitmarker then return end
	local safe_rect_pixels = managers.viewport:get_safe_rect_pixels()
	self._dorhud_hitmarker = {}
	self._dorhud_hitmarker.ws = Overlay:newgui():create_screen_workspace()
	self._dorhud_hitmarker._latest_update = nil
	self._dorhud_hitmarker.enabled = true
	self._dorhud_hitmarker.panel = self._dorhud_hitmarker.ws:panel()
	self._dorhud_hitmarker.panel:set_visible(false)
	local tex = "guis/textures/hud_icons"
	local tex_rect = {481, 33, 23, 4}
	local scale = DorHUD:conf("hitmarkers_scale") or 1
	self._dorhud_hitmarker.part_tl = self._dorhud_hitmarker.panel:bitmap({
		name = "hitmarker_tl",
		texture = tex,
		texture_rect = tex_rect,
		w = 17 * scale,
		h = 3 * scale,
		layer = 3,
		rotation = 315
	})
	self._dorhud_hitmarker.part_tr = self._dorhud_hitmarker.panel:bitmap({
		name = "hitmarker_tr",
		texture = tex,
		texture_rect = tex_rect,
		w = 17 * scale,
		h = 3 * scale,
		layer = 3,
		rotation = 225
	})
	self._dorhud_hitmarker.part_bl = self._dorhud_hitmarker.panel:bitmap({
		name = "hitmarker_bl",
		texture = tex,
		texture_rect = tex_rect,
		w = 17 * scale,
		h = 3 * scale,
		layer = 3,
		rotation = 135
	})
	self._dorhud_hitmarker.part_br = self._dorhud_hitmarker.panel:bitmap({
		name = "hitmarker_br",
		texture = tex,
		texture_rect = tex_rect,
		w = 17 * scale,
		h = 3 * scale,
		layer = 3,
		rotation = 45
	})
	self._dorhud_hitmarker.part_kill = self._dorhud_hitmarker.panel:bitmap({
		name = "hitmarker_kill",
		texture = "guis/textures/menu_radiobutton",
		texture_rect = {0, 0, 24, 24},
		w = 18 * scale,
		h = 18 * scale,
		layer = 2,
		visible = false
	})
	self._dorhud_hitmarker.part_tl:set_center(self._dorhud_hitmarker.panel:center())
	self._dorhud_hitmarker.part_tr:set_center(self._dorhud_hitmarker.panel:center())
	self._dorhud_hitmarker.part_bl:set_center(self._dorhud_hitmarker.panel:center())
	self._dorhud_hitmarker.part_br:set_center(self._dorhud_hitmarker.panel:center())
	self._dorhud_hitmarker.part_kill:set_center(self._dorhud_hitmarker.panel:center())
end)

module:hook(HUDManager, "resolution_changed", function(self)
	module:call_orig(HUDManager, "resolution_changed", self)
	if self._dorhud_hitmarker then
		Overlay:newgui():destroy_workspace(self._dorhud_hitmarker.ws)
		self._dorhud_hitmarker = nil
		self:setup_dorhud_hitmarkers_workspace()
	end
end)

module:hook(HUDManager, "confirm_hit", function(self, head, kill)
	local color = Color(DorHUD:conf("hitmarkers_hit_r") or 1, DorHUD:conf("hitmarkers_hit_g") or 0.4, DorHUD:conf("hitmarkers_hit_b") or 0.4):with_alpha(DorHUD:conf("hitmarkers_hit_a") or 0.5)
	if head then
		color = Color(DorHUD:conf("hitmarkers_head_r") or 1, DorHUD:conf("hitmarkers_head_g") or 0.6, DorHUD:conf("hitmarkers_head_b") or 0.1):with_alpha(DorHUD:conf("hitmarkers_head_a") or 0.5)
	end
	self._dorhud_hitmarker.panel:stop()
	self._dorhud_hitmarker.part_tl:set_color(color)
	self._dorhud_hitmarker.part_tr:set_color(color)
	self._dorhud_hitmarker.part_bl:set_color(color)
	self._dorhud_hitmarker.part_br:set_color(color)
	local rotation = 0
	if DorHUD:conf("hitmarkers_vibrate") then
		rotation = math.random(0, 89)
	elseif DorHUD:conf("hitmarkers_default_rotation") then
		rotation = DorHUD:conf("hitmarkers_default_rotation")
	end
	self._dorhud_hitmarker.part_tl:set_rotation(rotation)
	self._dorhud_hitmarker.part_tr:set_rotation(rotation + 90)
	self._dorhud_hitmarker.part_bl:set_rotation(rotation + 180)
	self._dorhud_hitmarker.part_br:set_rotation(rotation + 270)
	self._dorhud_hitmarker.part_kill:set_color(color)
	self._dorhud_hitmarker.part_kill:set_visible(kill and DorHUD:conf("hitmarkers_circle_on_kill"))
	self._dorhud_hitmarker.panel:animate(callback(self, self, "_animate_hit"), callback(self, self, "_animate_hit_done"), 0.25)
end)

module:hook(HUDManager, "_animate_hit", function(self, hint_confirm, done_cb, seconds)
	hint_confirm:set_visible(true)
	hint_confirm:set_alpha(1)
	--do
		--while 1 do
			local t = seconds
			while t > 0 do
				local dt = coroutine.yield()
				t = t - dt
				if DorHUD:conf("hitmarkers_rotate") then
					local rotation = math.floor(t * (DorHUD:conf("hitmarkers_rotation_speed") or 10) + hint_confirm:child("hitmarker_tl"):rotation() + 0.5) % 90
					hint_confirm:child("hitmarker_tl"):set_rotation(rotation)
					hint_confirm:child("hitmarker_tr"):set_rotation((rotation + 90))
					hint_confirm:child("hitmarker_bl"):set_rotation((rotation + 180))
					hint_confirm:child("hitmarker_br"):set_rotation((rotation + 270))
				end
				hint_confirm:set_alpha(t / seconds)
			end
			hint_confirm:set_visible(false)
			done_cb()
		--end
	--end
end)

module:hook(HUDManager, "_animate_hit_done", function(self)
	return -- dummy
end)
